@@ -6,13 +6,11 @@ from typing import TYPE_CHECKING, Any
 import discord
 from discord.embeds import Embed
 
-from spellbot.enums import GameFormat, GameService
+from spellbot.enums import GameBracket, GameFormat, GameService
 from spellbot.models import Channel, ChannelDict, GuildAward, GuildAwardDict
 from spellbot.operations import (
     safe_fetch_text_channel,
-    safe_get_partial_message,
     safe_send_channel,
-    safe_update_embed,
     safe_update_embed_origin,
 )
 from spellbot.services import GamesService
@@ -223,7 +221,7 @@ class AdminAction(BaseAction):
         if not found:
             return await self._report_failure()
 
-        embed = await games.to_embed(dm=True)
+        embed = await games.to_embed(guild=self.guild, dm=True)
         await safe_send_channel(self.interaction, embed=embed, ephemeral=True)
         return None
 
@@ -232,6 +230,29 @@ class AdminAction(BaseAction):
         view = SetupView(self.bot)
         await safe_send_channel(self.interaction, embed=embed, view=view)
 
+    async def setup_mythic_track(self) -> None:
+        assert self.guild is not None
+        enabled = await self.services.guilds.setup_mythic_track()
+        embed = Embed(title=f"Mythic Track Setup for {self.guild.name}")
+        embed.set_thumbnail(url=settings.ICO_URL)
+        embed.color = discord.Color(settings.INFO_EMBED_COLOR)
+        if enabled:
+            GUIDE = "https://www.mythictrack.com/spellbot"
+            embed.description = (
+                "✅ Mythic Track has been turned **on** for this server!\n\n"
+                "To continue setup, please "
+                f"[connect your Discord account to Mythic Track]({GUIDE}).\n\n"
+                "While this feature is turned on, SpellBot will generate PIN codes for "
+                "players to enter into Mythic Track so that their games can be tracked."
+            )
+        else:
+            embed.description = (
+                "❌ Mythic Track has been turned **off** for this server. Rerun this command "
+                "to toggle it back on. Note that turning this feature off does _not_ remove "
+                "any existing Mythic Track data."
+            )
+        await safe_send_channel(self.interaction, embed=embed)
+
     async def channels(self, page: int) -> None:
         embeds = await self._build_channels_embeds()
         try:
@@ -312,6 +333,27 @@ class AdminAction(BaseAction):
         embed.color = settings.INFO_EMBED_COLOR
         await safe_send_channel(self.interaction, embed=embed, ephemeral=True)
 
+    async def set_suggest_vc_category(self, category: str | None) -> None:
+        guild = await self.services.guilds.to_dict()
+        if guild["voice_create"]:
+            await safe_send_channel(
+                self.interaction,
+                (
+                    "Voice channel creation is enabled for this server. "
+                    "There's no need to suggest existing voice channels. "
+                    "New channels will be created automatically."
+                ),
+                ephemeral=True,
+            )
+            return
+
+        await self.services.guilds.set_suggest_vc_category(category)
+        if category:
+            msg = f'Suggested voice channels category prefix set to "{category}".'
+        else:
+            msg = "Suggested voice channels turned off."
+        await safe_send_channel(self.interaction, msg, ephemeral=True)
+
     async def set_motd(self, message: str | None = None) -> None:
         await self.services.guilds.set_motd(message)
         await safe_send_channel(self.interaction, "Message of the day updated.", ephemeral=True)
@@ -357,6 +399,15 @@ class AdminAction(BaseAction):
             ephemeral=True,
         )
 
+    async def set_default_bracket(self, bracket: int) -> None:
+        assert self.interaction.channel_id is not None
+        await self.services.channels.set_default_bracket(self.interaction.channel_id, bracket)
+        await safe_send_channel(
+            self.interaction,
+            f"Default bracket set to {GameBracket(bracket)} for this channel.",
+            ephemeral=True,
+        )
+
     async def set_default_service(self, service: int) -> None:
         assert self.interaction.channel_id is not None
         await self.services.channels.set_default_service(self.interaction.channel_id, service)
@@ -442,16 +493,17 @@ class AdminAction(BaseAction):
             ephemeral=True,
         )
 
-    async def set_require_confirmation(self, value: bool) -> None:
-        assert self.interaction.channel_id is not None
-        setting = await self.services.channels.set_require_confirmation(
-            self.interaction.channel_id, value
-        )
-        await safe_send_channel(
-            self.interaction,
-            f"Require confirmation setting for this channel has been set to: {setting}",
-            ephemeral=True,
-        )
+    # TODO: Refactor how confirmation/points/ELO works.
+    # async def set_require_confirmation(self, value: bool) -> None:
+    #     assert self.interaction.channel_id is not None
+    #     setting = await self.services.channels.set_require_confirmation(
+    #         self.interaction.channel_id, value
+    #     )
+    #     await safe_send_channel(
+    #         self.interaction,
+    #         f"Require confirmation setting for this channel has been set to: {setting}",
+    #         ephemeral=True,
+    #     )
 
     async def set_voice_invite(self, value: bool) -> None:
         assert self.interaction.channel_id is not None
@@ -474,40 +526,41 @@ class AdminAction(BaseAction):
             ephemeral=True,
         )
 
-    async def set_points(self, game_id: int, player_xid: int, points: int) -> None:
-        found = await self.services.games.select(game_id)
-        if not found:
-            await safe_send_channel(
-                self.interaction,
-                "There is no game with that ID.",
-                ephemeral=True,
-            )
-            return
-
-        if not await self.services.games.players_included(player_xid):
-            await safe_send_channel(
-                self.interaction,
-                f"User <@{player_xid}> did not play in game SB{found.get('id')}.",
-                ephemeral=True,
-            )
-            return
-
-        await self.services.games.add_points(player_xid, points)
-        await self.services.games.confirm_points(player_xid)
-
-        for post in found.get("posts", []):
-            guild_xid = post["guild_xid"]
-            channel_xid = post["channel_xid"]
-            channel = await safe_fetch_text_channel(self.bot, guild_xid, channel_xid)
-            if channel:
-                message_xid = post["message_xid"]
-                message = safe_get_partial_message(channel, guild_xid, message_xid)
-                if message:
-                    embed = await self.services.games.to_embed()
-                    await safe_update_embed(message, embed=embed)
-
-        await safe_send_channel(
-            self.interaction,
-            f"Points for <@{player_xid}> for game SB{game_id} set to {points}.",
-            ephemeral=True,
-        )
+    # TODO: Refactor how confirmation/points/ELO works.
+    # async def set_points(self, game_id: int, player_xid: int, points: int) -> None:
+    #     found = await self.services.games.select(game_id)
+    #     if not found:
+    #         await safe_send_channel(
+    #             self.interaction,
+    #             "There is no game with that ID.",
+    #             ephemeral=True,
+    #         )
+    #         return
+    #
+    #     if not await self.services.games.players_included(player_xid):
+    #         await safe_send_channel(
+    #             self.interaction,
+    #             f"User <@{player_xid}> did not play in game SB{found.get('id')}.",
+    #             ephemeral=True,
+    #         )
+    #         return
+    #
+    #     await self.services.games.add_points(player_xid, points)
+    #     await self.services.games.confirm_points(player_xid)
+    #
+    #     for post in found.get("posts", []):
+    #         guild_xid = post["guild_xid"]
+    #         channel_xid = post["channel_xid"]
+    #         channel = await safe_fetch_text_channel(self.bot, guild_xid, channel_xid)
+    #         if channel:
+    #             message_xid = post["message_xid"]
+    #             message = safe_get_partial_message(channel, guild_xid, message_xid)
+    #             if message:
+    #                 embed = await self.services.games.to_embed(self.guild)
+    #                 await safe_update_embed(message, embed=embed)
+    #
+    #     await safe_send_channel(
+    #         self.interaction,
+    #         f"Points for <@{player_xid}> for game SB{game_id} set to {points}.",
+    #         ephemeral=True,
+    #     )
