@@ -1,17 +1,22 @@
 from __future__ import annotations
 
+import asyncio
+import json
 import logging
 import re
+import urllib.parse
 from typing import TYPE_CHECKING
 
 import discord
-from ddtrace import tracer
+from ddtrace.trace import tracer
 
-from spellbot.enums import GameFormat, GameService
-from spellbot.models import GameStatus
+from spellbot.enums import GameBracket, GameFormat, GameService
+from spellbot.models import MAX_RULES_LENGTH, GameStatus
 from spellbot.operations import (
+    VoiceChannelSuggestion,
     safe_add_role,
     safe_channel_reply,
+    safe_create_channel_invite,
     safe_create_voice_channel,
     safe_ensure_voice_category,
     safe_fetch_text_channel,
@@ -19,12 +24,12 @@ from spellbot.operations import (
     safe_followup_channel,
     safe_get_partial_message,
     safe_send_user,
+    safe_suggest_voice_channel,
     safe_update_embed,
     safe_update_embed_origin,
-    save_create_channel_invite,
 )
 from spellbot.settings import settings
-from spellbot.views import BaseView, PendingGameView, StartedGameView, StartedGameViewWithConfirm
+from spellbot.views import BaseView, PendingGameView, StartedGameView
 
 from .base_action import BaseAction
 
@@ -32,6 +37,7 @@ if TYPE_CHECKING:
     from discord.message import Message
 
     from spellbot import SpellBot
+    from spellbot.models import GameDict
 
 logger = logging.getLogger(__name__)
 
@@ -70,6 +76,18 @@ class LookingForGameAction(BaseAction):
             return self.channel_data["default_format"].value
         return GameFormat.COMMANDER.value
 
+    @tracer.wrap()
+    async def get_bracket(self, format: int | None, bracket: int | None = None) -> int:
+        if format == GameFormat.CEDH.value:
+            return GameBracket.BRACKET_5.value
+        if format == GameFormat.PRE_CONS.value:
+            return GameBracket.BRACKET_2.value
+        if bracket is not None:
+            return bracket
+        if self.channel_data["default_bracket"] is not None:
+            return self.channel_data["default_bracket"].value
+        return GameBracket.NONE.value
+
     @tracer.wrap()
     async def filter_friend_xids(self, friend_xids: list[int]) -> list[int]:
         assert self.guild
@@ -89,10 +107,13 @@ class LookingForGameAction(BaseAction):
         self,
         friend_xids: list[int],
         seats: int,
+        rules: str | None,
         format: int,
+        bracket: int,
         service: int,
         message_xid: int | None,
     ) -> bool | None:
+        """Return True if the game is new, False if existing, or None of user can not join."""
         assert self.guild
         assert self.channel
 
@@ -104,16 +125,16 @@ class LookingForGameAction(BaseAction):
 
         if not origin:
             assert self.interaction.guild_id is not None
-            mirrors = await self.services.mirrors.get(self.guild.id, self.channel.id)
             return await self.services.games.upsert(
                 guild_xid=self.interaction.guild_id,
                 channel_xid=self.channel.id,
                 author_xid=self.interaction.user.id,
                 friends=friend_xids,
                 seats=seats,
+                rules=rules,
                 format=format,
+                bracket=bracket,
                 service=service,
-                mirrors=mirrors,
             )
 
         assert message_xid
@@ -122,46 +143,67 @@ class LookingForGameAction(BaseAction):
             await safe_send_user(self.interaction.user, "You can not join this game.")
             return None
 
-        # Sometimes game posts have Join/Leave buttons on them even though
-        # the game has started. This can happen if an interaction fails on
-        # Discord's side of things. This makes it appear like a user can still
-        # join a game, even though it's already started. We need to handle this
-        # by informing the user and updating the game post they tried to join.
         if found["status"] == GameStatus.STARTED.value:
+            logger.warning("User tried to join a game that has already started.")
             # inform the player that their interaction failed
             await safe_send_user(
                 self.interaction.user,
                 "Sorry, that game has already started.",
             )
-
-            # attempt to update the problematic game post
-            if message := safe_get_partial_message(
-                self.channel,
-                self.guild.id,
-                message_xid,
-            ):
-                embed = await self.services.games.to_embed()
-                view: BaseView | None = None
-                if self.channel_data.get("show_points", False) and not found["confirmed"]:
-                    if self.channel_data.get("require_confirmation", False):
-                        view = StartedGameViewWithConfirm(bot=self.bot)
-                    else:
-                        view = StartedGameView(bot=self.bot)
-                await safe_update_embed(message, embed=embed, view=view)
-
             return None
 
         await self.services.games.add_player(self.interaction.user.id)
         return False
 
     @tracer.wrap()
-    async def execute(  # noqa: C901
+    async def execute_rematch(self) -> None:
+        if not self.guild or not self.channel:
+            # Someone tried to lfg in a Discord thread rather than in the channel itself.
+            await safe_send_user(
+                self.interaction.user,
+                "Please run this command in the same channel as your last played game.",
+            )
+            return
+
+        if await self.services.users.pending_games():
+            await safe_followup_channel(
+                self.interaction,
+                "You're already in a pending game, leave that one first.",
+            )
+            return
+
+        game_data = await self.services.games.select_last_game(
+            user_xid=self.interaction.user.id,
+            guild_xid=self.guild.id,
+        )
+        if not game_data:
+            await safe_followup_channel(
+                self.interaction,
+                "You have not played a game in this guild yet.",
+            )
+            return
+
+        player_xids = await self.services.games.player_xids()
+        players = " ".join(f"<@{xid}>" for xid in player_xids)
+
+        await self.create_game(
+            players=players,
+            format=game_data["format"],
+            bracket=game_data["bracket"],
+            service=GameService.NOT_ANY.value,
+            rematch=True,
+        )
+
+    @tracer.wrap()
+    async def execute(
         self,
         friends: str | None = None,
         seats: int | None = None,
+        rules: str | None = None,
         format: int | None = None,
-        message_xid: int | None = None,
+        bracket: int | None = None,
         service: int | None = None,
+        message_xid: int | None = None,
     ) -> None:
         if not self.guild or not self.channel:
             # Someone tried to lfg in a Discord thread rather than in the channel itself.
@@ -177,8 +219,10 @@ class LookingForGameAction(BaseAction):
 
         actual_friends: str = await self.get_friends(friends)
         actual_format: int = await self.get_format(format)
+        actual_bracket: int = await self.get_bracket(actual_format, bracket)
         actual_seats: int = await self.get_seats(actual_format, seats)
         actual_service: int = await self.get_service(service)
+        rules = None if not rules else rules[:MAX_RULES_LENGTH]
 
         if await self.services.users.is_waiting(self.channel.id):
             msg = "You're already in a game in this channel."
@@ -187,14 +231,6 @@ class LookingForGameAction(BaseAction):
             await safe_followup_channel(self.interaction, msg)
             return None
 
-        req_confirm = self.channel_data["require_confirmation"]
-        if req_confirm and not await self.services.users.is_confirmed(self.channel.id):
-            msg = "You need to confirm your points before joining another game."
-            if origin:
-                return await safe_send_user(self.interaction.user, msg)
-            await safe_followup_channel(self.interaction, msg)
-            return None
-
         if await self.services.users.pending_games() + 1 > settings.MAX_PENDING_GAMES:
             msg = "You're in too many pending games to join another one at this time."
             if origin:
@@ -216,7 +252,9 @@ class LookingForGameAction(BaseAction):
         new = await self.upsert_game(
             friend_xids,
             actual_seats,
+            rules,
             actual_format,
+            actual_bracket,
             actual_service,
             message_xid,
         )
@@ -226,20 +264,24 @@ class LookingForGameAction(BaseAction):
         fully_seated = await self.services.games.fully_seated()
         started_game_id: int | None = None
         other_game_ids: list[int] = []
+        suggested_vc = None
         if fully_seated:
             other_game_ids = await self.services.games.other_game_ids()
-            started_game_id = await self.make_game_ready(GameService(actual_service))
+            game_data = await self.services.games.to_dict()
+            player_xids = await self.services.games.player_xids()
+            started_game_id, suggested_vc = await self.make_game_ready(game_data, player_xids)
             await self._handle_voice_creation(self.guild.id)
 
         await self._handle_embed_creation(
             new=new,
             origin=origin,
             fully_seated=fully_seated,
+            suggested_vc=suggested_vc,
         )
 
         if fully_seated:
             assert started_game_id is not None
-            await self._handle_direct_messages()
+            await self._handle_direct_messages(suggested_vc=suggested_vc)
             await self._update_other_game_posts(other_game_ids)
             return None
         return None
@@ -267,7 +309,7 @@ class LookingForGameAction(BaseAction):
                     message_xid,
                 )
             ):
-                embed = await self.services.games.to_embed()
+                embed = await self.services.games.to_embed(guild=self.guild)
                 await safe_update_embed(
                     message,
                     embed=embed,
@@ -287,81 +329,23 @@ class LookingForGameAction(BaseAction):
             )
             return
 
-        plays = await self.services.games.get_plays()
-        if plays.get(self.interaction.user.id, {}).get("confirmed_at", None):
-            await safe_send_user(
-                self.interaction.user,
-                f"You've already confirmed your points for game SB{found.get('id')}.",
-            )
-            return
-
-        # if at least one player has confirmed their points, then changing points not allowed
-        if any(play.get("confirmed_at") is not None for play in plays.values()):
-            await safe_send_user(
-                self.interaction.user,
-                (
-                    f"Points for game SB{found.get('id')} are locked in,"
-                    " please confirm them or contact a mod."
-                ),
-            )
-            return
-
         await self.services.games.add_points(self.interaction.user.id, points)
-        embed = await self.services.games.to_embed()
+        embed = await self.services.games.to_embed(guild=self.guild)
         await safe_update_embed(message, embed=embed)
 
-    @tracer.wrap()
-    async def confirm_points(self, message: Message) -> None:
-        found = await self.services.games.select_by_message_xid(message.id)
-        if not found:
-            return
-
-        if not await self.services.games.players_included(self.interaction.user.id):
-            await safe_send_user(
-                self.interaction.user,
-                f"You are not one of the players in game SB{found.get('id')}.",
-            )
-            return
-
-        plays = await self.services.games.get_plays()
-        if plays.get(self.interaction.user.id, {}).get("confirmed_at", None):
-            await safe_send_user(
-                self.interaction.user,
-                f"You've already confirmed your points for game SB{found.get('id')}",
-            )
-            return
-
-        if any(play.get("points") is None for play in plays.values()):
-            await safe_send_user(
-                self.interaction.user,
-                (
-                    "Please wait until all players have reported"
-                    f" before confirming points for game SB{found.get('id')}"
-                ),
-            )
-            return
-
-        confirmed_at = await self.services.games.confirm_points(player_xid=self.interaction.user.id)
-        embed = await self.services.games.to_embed()
-        data = await self.services.games.to_dict()
-        if data["confirmed"]:
-            await safe_update_embed(message, embed=embed, view=None)
-        else:
-            await safe_update_embed(message, embed=embed)
-        plays[self.interaction.user.id]["confirmed_at"] = confirmed_at
-        if all(play["confirmed_at"] is not None for play in plays.values()):
-            await self.services.games.update_records(plays)
-
     @tracer.wrap()
     async def create_game(
         self,
         players: str,
         format: int | None = None,
+        bracket: int | None = None,
         service: int | None = None,
+        rematch: bool = False,
     ) -> None:
         assert self.channel
 
         game_format = GameFormat(format or GameFormat.COMMANDER.value)
+        game_bracket = GameBracket(bracket or GameBracket.NONE.value)
         game_service = GameService(service or GameService.SPELLTABLE.value)
         player_xids = list(map(int, re.findall(r"<@!?(\d+)>", players)))
         requested_seats = len(player_xids)
@@ -394,21 +378,59 @@ class LookingForGameAction(BaseAction):
             author_xid=found_players[0],
             friends=found_players[1:],
             seats=requested_seats,
+            rules=None,
             format=game_format.value,
+            bracket=game_bracket.value,
             service=game_service.value,
             create_new=True,
         )
-        await self.make_game_ready(game_service)
+        game_data = await self.services.games.to_dict()
+        _, suggested_vc = await self.make_game_ready(game_data, player_xids)
         await self._handle_voice_creation(self.interaction.guild_id)
-        await self._handle_embed_creation(new=True, origin=False, fully_seated=True)
-        await self._handle_direct_messages()
+        await self._handle_embed_creation(
+            new=True,
+            origin=False,
+            fully_seated=True,
+            suggested_vc=suggested_vc,
+            rematch=rematch,
+        )
+        await self._handle_direct_messages(suggested_vc=suggested_vc, rematch=rematch)
 
     @tracer.wrap()
-    async def make_game_ready(self, service: GameService) -> int:
-        spelltable_link = (
-            await self.bot.create_spelltable_link() if service == GameService.SPELLTABLE else None
-        )
-        return await self.services.games.make_ready(spelltable_link)
+    async def make_game_ready(
+        self,
+        game: GameDict,
+        player_xids: list[int],
+    ) -> tuple[int, VoiceChannelSuggestion | None]:
+        details = await self.bot.create_game_link(game)
+
+        suggested_vc = None
+        if (
+            not game["voice_xid"]
+            and not game["voice_invite_link"]
+            and self.guild_data
+            and (suggest_voice_category := self.guild_data["suggest_voice_category"])
+            and self.guild is not None
+        ):
+            suggested_vc = safe_suggest_voice_channel(
+                guild=self.guild,
+                category=suggest_voice_category,
+                player_xids=player_xids,
+            )
+
+        if span := tracer.current_span():  # pragma: no cover
+            span.set_tags(
+                {
+                    "game_id": str(game["id"]),
+                    "voice_xid": str(game["voice_xid"]),
+                    "voice_invite_link": str(game["voice_invite_link"]),
+                    "guild_data__isnull": str(bool(self.guild_data is None)),
+                    "already_picked": str(suggested_vc.already_picked if suggested_vc else None),
+                    "random_empty": str(suggested_vc.random_empty if suggested_vc else None),
+                }
+            )
+
+        return await self.services.games.make_ready(details.link, details.password), suggested_vc
 
     @tracer.wrap()
     async def _handle_voice_creation(self, guild_xid: int) -> None:
@@ -440,7 +462,7 @@ class LookingForGameAction(BaseAction):
         should_create_invite = self.channel_data.get("voice_invite", False)
         invite: discord.Invite | None = None
         if should_create_invite:
-            invite = await save_create_channel_invite(
+            invite = await safe_create_channel_invite(
                 voice_channel,
                 max_age=2 * 60 * 60,  # 2 hours
                 max_uses=0,  # unlimited uses
@@ -462,7 +484,6 @@ class LookingForGameAction(BaseAction):
     ) -> None:
         assert self.guild
         assert self.channel
-
         if message := await safe_followup_channel(
             self.interaction,
             content=content,
@@ -478,50 +499,32 @@ class LookingForGameAction(BaseAction):
         ):
             await self.services.games.add_post(self.guild.id, self.channel.id, message.id)
 
-        # also send the game post to all configured mirrors
-        mirrors = await self.services.mirrors.get(self.guild.id, self.channel.id)
-        for mirror in mirrors:
-            to_guild_xid = mirror["to_guild_xid"]
-            to_channel_xid = mirror["to_channel_xid"]
-            logger.info("Mirroring game post to %s/%s ...", to_guild_xid, to_channel_xid)
-
-            to_channel = await safe_fetch_text_channel(self.bot, to_guild_xid, to_channel_xid)
-            if to_channel is None:
-                logger.error("Failed to fetch channel %s", to_channel_xid)
-                continue
-            logger.info("Mirroring game post to %s ...", to_channel)
-
-            to_message = await safe_channel_reply(
-                to_channel, content=content, embed=embed, view=view
-            )
-            if to_message is None:
-                logger.error("Failed to create post in channel %s", to_channel)
-                continue
-            logger.info("Mirrored game post to %s", to_message)
-            await self.services.games.add_post(to_guild_xid, to_channel_xid, to_message.id)
-
     @tracer.wrap()
     async def _handle_embed_creation(  # noqa: C901,PLR0912
         self,
+        *,
         new: bool,
         origin: bool,
         fully_seated: bool,
+        suggested_vc: VoiceChannelSuggestion | None = None,
+        rematch: bool = False,
     ) -> None:
         assert self.guild
         assert self.channel
 
         # build the game post's embed and view:
-        embed: discord.Embed = await self.services.games.to_embed()
+        embed: discord.Embed = await self.services.games.to_embed(
+            guild=self.guild,
+            suggested_vc=suggested_vc,
+            rematch=rematch,
+        )
         content = self.channel_data.get("extra", None)
         game_data = await self.services.games.to_dict()
 
         view: BaseView | None = None
         if fully_seated:
-            if self.channel_data.get("show_points", False) and not game_data["confirmed"]:
-                if self.channel_data.get("require_confirmation", False):
-                    view = StartedGameViewWithConfirm(bot=self.bot)
-                else:
-                    view = StartedGameView(bot=self.bot)
+            if self.channel_data.get("show_points", False):
+                view = StartedGameView(bot=self.bot)
         else:
             view = PendingGameView(bot=self.bot)
 
@@ -580,29 +583,82 @@ class LookingForGameAction(BaseAction):
         embed.set_thumbnail(url=settings.ICO_URL)
         game_data = await self.services.games.to_dict()
         links = game_data["jump_links"]
-        link = links[self.guild.id]
         format = game_data["format"]
         embed.set_author(name=f"I found a {GameFormat(format)} game for you!")
-        embed.description = f"You can [jump to the game post]({link}) to see it!"
         embed.color = settings.INFO_EMBED_COLOR
+        if link := links.get(self.guild.id):
+            embed.description = f"You can [jump to the game post]({link}) to see it!"
+        else:
+            game_id = game_data["id"]
+            embed.description = f"You have joined the game SB{game_id}!"
         await safe_followup_channel(self.interaction, embed=embed)
 
     @tracer.wrap()
-    async def _handle_direct_messages(self) -> None:
-        player_xids = await self.services.games.player_xids()
-        embed = await self.services.games.to_embed(dm=True)
-        failed_xids: list[int] = []
+    async def _handle_direct_messages(
+        self,
+        suggested_vc: VoiceChannelSuggestion | None = None,
+        rematch: bool = False,
+    ) -> None:
+        player_pins = await self.services.games.player_pins()
+        player_names = await self.services.games.player_names()
+        player_xids = list(player_pins.keys())
+        game_data = await self.services.games.to_dict()
+        base_embed = await self.services.games.to_embed(
+            guild=self.guild,
+            dm=True,
+            suggested_vc=suggested_vc,
+            rematch=rematch,
+        )
 
         # notify players
         fetched_players: dict[int, discord.User] = {}
-        for player_xid in player_xids:
-            player = await safe_fetch_user(self.bot, player_xid)
-            if player:
+        failed_xids: list[int] = []
+
+        def mythic_track_link(player_xid: int) -> str:
+            assert self.guild
+            players_data = urllib.parse.quote(
+                json.dumps(
+                    [
+                        {
+                            "user_xid": f"{pid}",
+                            "username": pn,
+                        }
+                        for pid, pn in player_names.items()
+                    ],
+                    separators=(",", ":"),
+                ),
+                safe="",
+                encoding=None,
+                errors=None,
+            )
+            return (
+                "https://www.mythictrack.com/gamelobby"
+                f"/{self.guild.id}"
+                f"/{game_data['id']}"
+                f"/{game_data['format']}"
+                f"/{game_data['service']}"
+                f"/{player_xid}"
+                f"/{players_data}"
+                f"/{game_data['bracket'] - 1}"
+            )
+
+        async def notify_player(player_xid: int) -> None:
+            embed = base_embed.copy()
+            if pin := player_pins[player_xid]:
+                embed.description = (
+                    f"{embed.description}\n\n"
+                    f"Track your game on [Mythic Track]({mythic_track_link(player_xid)}) "
+                    f"with PIN code: `{pin}`"
+                )
+            if player := await safe_fetch_user(self.bot, player_xid):
                 fetched_players[player_xid] = player
                 await safe_send_user(player, embed=embed)
             else:
                 failed_xids.append(player_xid)
 
+        notify_player_tasks = [notify_player(player_xid) for player_xid in player_xids]
+        await asyncio.gather(*notify_player_tasks)
+
         # give out awards
         assert self.interaction.guild_id is not None
         new_roles = await self.services.awards.give_awards(
