@@ -1,11 +1,9 @@
-# pylint: disable=too-many-arguments,too-many-locals,too-many-statements
 from __future__ import annotations
 
 import asyncio
 import time
 from os import getenv
 from socket import socket
-from typing import Optional
 
 import click
 import hupper
@@ -16,12 +14,14 @@ from . import __version__
 from .environment import running_in_pytest
 from .logs import configure_logging
 from .metrics import no_metrics
+from .settings import settings
 
 # load .env environment variables as early as possible
 if not running_in_pytest():  # pragma: no cover
     load_dotenv()
 
-uvloop.install()
+if not getenv("DISABLE_UVLOOP", ""):  # pragma: no cover
+    uvloop.install()
 
 
 @click.command()
@@ -69,20 +69,17 @@ uvloop.install()
 )
 @click.version_option(version=__version__)
 def main(
-    log_level: Optional[str],
+    log_level: str | None,
     dev: bool,
     debug: bool,
     mock_games: bool,
     api: bool,
-    port: Optional[int] = None,
+    port: int | None = None,
 ) -> None:
     if dev:
         hupper.start_reloader("spellbot.main")
 
-    from .settings import Settings
-
     # Ensure that configure_logging() is called as early as possible
-    settings = Settings()
     level = log_level if log_level is not None else (getenv("LOG_LEVEL") or "INFO")
     configure_logging(level)
 
@@ -96,7 +93,7 @@ def main(
     # When metrics are enabled, let's ensure that datadog-agent is running first...
     if not no_metrics():  # pragma: no cover
         logger = logging.root
-        conn: Optional[socket] = None
+        conn: socket | None = None
         connected = False
 
         logger.info("metrics enabled, checking for connection to statsd server...")
@@ -120,7 +117,7 @@ def main(
 
         loop = asyncio.new_event_loop()
         loop.set_debug(debug)
-        launch_web_server(settings, loop, port or settings.PORT)
+        launch_web_server(loop, port or settings.PORT)
         loop.run_forever()
     else:
         from .client import build_bot
