@@ -1,14 +1,14 @@
 import logging
 
 import discord
-from ddtrace import tracer
+from ddtrace.trace import tracer
 from discord import app_commands
 from discord.app_commands import Choice
 from discord.ext import commands
 
 from spellbot import SpellBot
 from spellbot.actions import AdminAction
-from spellbot.enums import GameFormat, GameService
+from spellbot.enums import GAME_BRACKET_ORDER, GAME_FORMAT_ORDER, GAME_SERVICE_ORDER
 from spellbot.metrics import add_span_context
 from spellbot.settings import settings
 from spellbot.utils import for_all_callbacks, is_admin, is_guild
@@ -29,6 +29,16 @@ class AdminCog(commands.Cog):
         async with AdminAction.create(self.bot, interaction) as action:
             await action.setup()
 
+    @app_commands.command(
+        name="setup_mythic_track",
+        description="Setup Mythic Track on your server.",
+    )
+    @tracer.wrap(name="interaction", resource="setup")
+    async def setup_mythic_track(self, interaction: discord.Interaction) -> None:
+        add_span_context(interaction)
+        async with AdminAction.create(self.bot, interaction) as action:
+            await action.setup_mythic_track()
+
     @app_commands.command(name="forget_channel", description="Forget settings for a channel.")
     @app_commands.describe(channel="What is the Discord ID of the channel?")
     @tracer.wrap(name="interaction", resource="forget_channel")
@@ -39,6 +49,21 @@ class AdminCog(commands.Cog):
 
     set_group = app_commands.Group(name="set", description="...")
 
+    @set_group.command(
+        name="suggest_vc_category",
+        description="Set the category prefix to use for suggested voice channels.",
+    )
+    @app_commands.describe(category="Category prefix")
+    @tracer.wrap(name="interaction", resource="set_suggest_vc_category")
+    async def set_suggest_vc_category(
+        self,
+        interaction: discord.Interaction,
+        category: str | None = None,
+    ) -> None:
+        add_span_context(interaction)
+        async with AdminAction.create(self.bot, interaction) as action:
+            await action.set_suggest_vc_category(category)
+
     @set_group.command(
         name="motd",
         description="Set your server's message of the day. Leave blank to unset.",
@@ -168,7 +193,7 @@ class AdminCog(commands.Cog):
     )
     @app_commands.describe(format="Default game format")
     @app_commands.choices(
-        format=[Choice(name=str(format), value=format.value) for format in GameFormat],
+        format=[Choice(name=str(format), value=format.value) for format in GAME_FORMAT_ORDER],
     )
     @tracer.wrap(name="interaction", resource="set_default_format")
     async def default_format(self, interaction: discord.Interaction, format: int) -> None:
@@ -176,13 +201,27 @@ class AdminCog(commands.Cog):
         async with AdminAction.create(self.bot, interaction) as action:
             await action.set_default_format(format)
 
+    @set_group.command(
+        name="default_bracket",
+        description="Set the default commander bracket for new games in this channel.",
+    )
+    @app_commands.describe(bracket="Default commander bracket")
+    @app_commands.choices(
+        bracket=[Choice(name=str(bracket), value=bracket.value) for bracket in GAME_BRACKET_ORDER],
+    )
+    @tracer.wrap(name="interaction", resource="set_default_format")
+    async def default_bracket(self, interaction: discord.Interaction, bracket: int) -> None:
+        add_span_context(interaction)
+        async with AdminAction.create(self.bot, interaction) as action:
+            await action.set_default_bracket(bracket)
+
     @set_group.command(
         name="default_service",
         description="Set the default game service for new games in this channel.",
     )
     @app_commands.describe(service="Default service")
     @app_commands.choices(
-        service=[Choice(name=str(service), value=service.value) for service in GameService],
+        service=[Choice(name=str(service), value=service.value) for service in GAME_SERVICE_ORDER],
     )
     @tracer.wrap(name="interaction", resource="set_default_service")
     async def default_service(self, interaction: discord.Interaction, service: int) -> None:
@@ -264,16 +303,17 @@ class AdminCog(commands.Cog):
         async with AdminAction.create(self.bot, interaction) as action:
             await action.set_show_points(setting)
 
-    @set_group.command(
-        name="require_confirmation",
-        description="Set the option for requiring points confirmation on games in this channel.",
-    )
-    @app_commands.describe(setting="Setting")
-    @tracer.wrap(name="interaction", resource="set_require_confirmation")
-    async def require_confirmation(self, interaction: discord.Interaction, setting: bool) -> None:
-        add_span_context(interaction)
-        async with AdminAction.create(self.bot, interaction) as action:
-            await action.set_require_confirmation(setting)
+    # TODO: I need to completely rework how the ELO system works.
+    # @set_group.command(
+    #     name="require_confirmation",
+    #     description="Set the option for requiring points confirmation on games in this channel.",
+    # )
+    # @app_commands.describe(setting="Setting")
+    # @tracer.wrap(name="interaction", resource="set_require_confirmation")
+    # async def require_confirmation(self, interaction: discord.Interaction, setting: bool) -> None:
+    #     add_span_context(interaction)
+    #     async with AdminAction.create(self.bot, interaction) as action:
+    #         await action.set_require_confirmation(setting)
 
     @set_group.command(
         name="voice_invite",
