@@ -12,7 +12,7 @@ class FormatDetails(NamedTuple):
 class GameService(Enum):
     """A service for playing Magic: The Gathering games."""
 
-    def __new__(cls, *args: Any, **kwargs: Any) -> Any:
+    def __new__(cls, *args: Any, **kwargs: Any) -> Any:  # noqa: ARG004
         """Give each enum value an increasing numerical value starting at 1."""
         value = len(cls.__members__) + 1
         obj = object.__new__(cls)
@@ -25,6 +25,7 @@ class GameService(Enum):
     def __str__(self) -> str:
         return self.title
 
+    # DO NOT REORDER -- IT WOULD INVALIDATE EXISTING DATABASE ENTRIES!
     NOT_ANY = "Not any"
     SPELLTABLE = "SpellTable"
     COCKATRICE = "Cockatrice"
@@ -32,12 +33,25 @@ class GameService(Enum):
     MTG_ARENA = "MTG Arena"
     MTG_ONLINE = "MTG Online"
     TTS = "TabletopSim"
+    TABLE_STREAM = "Table Stream"
+
+
+GAME_SERVICE_ORDER = [
+    GameService.NOT_ANY,
+    GameService.SPELLTABLE,
+    GameService.TABLE_STREAM,
+    GameService.COCKATRICE,
+    GameService.X_MAGE,
+    GameService.MTG_ARENA,
+    GameService.MTG_ONLINE,
+    GameService.TTS,
+]
 
 
 class GameFormat(Enum):
     """A Magic: The Gathering game format."""
 
-    def __new__(cls, *args: Any, **kwargs: Any) -> Any:
+    def __new__(cls, *args: Any, **kwargs: Any) -> Any:  # noqa: ARG004
         """Give each enum value an increasing numerical value starting at 1."""
         value = len(cls.__members__) + 1
         obj = object.__new__(cls)
@@ -74,9 +88,10 @@ class GameFormat(Enum):
     OATHBREAKER = 4
     DUEL_COMMANDER = 2
     CEDH = 4, "cEDH"
+    ARCHENEMY = 4
+    PAUPER_EDH = 4, "Pauper EDH"
 
 
-# Determines the order the formats appear in the /lfg command:
 GAME_FORMAT_ORDER = [
     GameFormat.COMMANDER,
     GameFormat.PRE_CONS,
@@ -89,7 +104,9 @@ GAME_FORMAT_ORDER = [
     GameFormat.TWO_HEADED_GIANT,
     GameFormat.BRAWL_MULTIPLAYER,
     GameFormat.CEDH,
+    GameFormat.PAUPER_EDH,
     GameFormat.OATHBREAKER,
+    GameFormat.ARCHENEMY,
     GameFormat.DUEL_COMMANDER,
     GameFormat.STANDARD,
     GameFormat.MODERN,
@@ -100,3 +117,38 @@ GAME_FORMAT_ORDER = [
     GameFormat.SEALED,
     GameFormat.BRAWL_TWO_PLAYER,
 ]
+
+
+class GameBracket(Enum):
+    """The bracket for this game."""
+
+    def __new__(cls, *args: Any, **kwargs: Any) -> Any:  # noqa: ARG004
+        """Give each enum value an increasing numerical value starting at 1."""
+        value = len(cls.__members__) + 1
+        obj = object.__new__(cls)
+        obj._value_ = value
+        return obj
+
+    def __init__(self, title: str) -> None:
+        self.title = title
+
+    def __str__(self) -> str:
+        return self.title
+
+    # DO NOT REORDER -- IT WOULD INVALIDATE EXISTING DATABASE ENTRIES!
+    NONE = "None"
+    BRACKET_1 = "Bracket 1: Exhibition"
+    BRACKET_2 = "Bracket 2: Core"
+    BRACKET_3 = "Bracket 3: Upgraded"
+    BRACKET_4 = "Bracket 4: Optimized"
+    BRACKET_5 = "Bracket 5: Competitive"
+
+
+GAME_BRACKET_ORDER = [
+    GameBracket.NONE,
+    GameBracket.BRACKET_1,
+    GameBracket.BRACKET_2,
+    GameBracket.BRACKET_3,
+    GameBracket.BRACKET_4,
+    GameBracket.BRACKET_5,
+]
