@@ -1,23 +1,30 @@
 from __future__ import annotations
 
-from datetime import datetime
+from dataclasses import dataclass
+from datetime import UTC, datetime
 from enum import Enum, auto
+from functools import partial
 from typing import TYPE_CHECKING, TypedDict, cast
 
 import discord
 from dateutil import tz
+from ddtrace.trace import tracer
 from sqlalchemy import BigInteger, Boolean, Column, DateTime, ForeignKey, Integer, String
 from sqlalchemy.orm import relationship
 from sqlalchemy.sql.expression import false, text
 
-from spellbot.enums import GameFormat, GameService
+from spellbot.enums import GameBracket, GameFormat, GameService
 from spellbot.settings import settings
 
 from . import Base, now
 
 if TYPE_CHECKING:
+    from spellbot.operations import VoiceChannelSuggestion
+
     from . import Channel, Guild, Post, PostDict, User  # noqa: F401
 
+HR = "**˙ॱ⋅.˳.⋅ॱ˙ॱ⋅.˳.⋅ॱ˙ॱ⋅.˳.⋅ॱ˙ॱ⋅.˳.⋅ॱ˙ॱ⋅.˳.⋅ॱ˙ॱ⋅.˳.⋅ॱ˙ॱ⋅.˳.⋅ॱ˙ॱ⋅.˳.⋅ॱ˙ॱ⋅.˳.⋅ॱ˙**"
+
 
 class GameStatus(Enum):
     PENDING = auto()
@@ -38,12 +45,21 @@ class GameDict(TypedDict):
     seats: int
     status: int
     format: int
+    bracket: int
     service: int
     spelltable_link: str | None
     spectate_link: str | None
     jump_links: dict[int, str]
     confirmed: bool
     requires_confirmation: bool
+    password: str | None
+    rules: str | None
+
+
+@dataclass
+class GameLinkDetails:
+    link: str | None = None
+    password: str | None = None
 
 
 class Game(Base):
@@ -61,16 +77,17 @@ class Game(Base):
     created_at = Column(
         DateTime,
         nullable=False,
-        default=datetime.utcnow,
+        default=partial(datetime.now, UTC),
         server_default=now,
         doc="UTC timestamp when this games was first created",
     )
     updated_at = Column(
         DateTime,
         nullable=False,
-        default=datetime.utcnow,
+        default=partial(datetime.now, UTC),
         server_default=now,
-        onupdate=datetime.utcnow,
+        onupdate=partial(datetime.now, UTC),
+        index=True,
         doc="UTC timestamp when this games was last updated",
     )
     started_at = Column(
@@ -92,7 +109,7 @@ class Game(Base):
         doc="The external Discord ID of the associated guild",
     )
     channel_xid: int = cast(
-        int,
+        "int",
         Column(
             BigInteger,
             ForeignKey("channels.xid", ondelete="CASCADE"),
@@ -108,7 +125,7 @@ class Game(Base):
         doc="The external Discord ID of an associated voice channel",
     )
     seats: int = cast(
-        int,
+        "int",
         Column(
             Integer,
             index=True,
@@ -117,7 +134,7 @@ class Game(Base):
         ),
     )
     status: int = cast(
-        int,
+        "int",
         Column(
             Integer(),
             default=GameStatus.PENDING.value,
@@ -128,7 +145,7 @@ class Game(Base):
         ),
     )
     format: int = cast(
-        int,
+        "int",
         Column(
             Integer(),
             default=GameFormat.COMMANDER.value,
@@ -138,8 +155,19 @@ class Game(Base):
             doc="The Magic: The Gathering format for this game",
         ),
     )
+    bracket: int = cast(
+        "int",
+        Column(
+            Integer(),
+            default=GameBracket.NONE.value,
+            server_default=text(str(GameBracket.NONE.value)),
+            index=True,
+            nullable=False,
+            doc="The commander bracket for this game",
+        ),
+    )
     service: int = cast(
-        int,
+        "int",
         Column(
             Integer(),
             default=GameService.SPELLTABLE.value,
@@ -150,6 +178,7 @@ class Game(Base):
         ),
     )
     spelltable_link = Column(String(255), doc="The generated SpellTable link for this game")
+    password = Column(String(255), nullable=True, doc="The password for this game")
     voice_invite_link = Column(String(255), doc="The voice channel invite link for this game")
     requires_confirmation = Column(
         Boolean,
@@ -158,6 +187,7 @@ class Game(Base):
         server_default=false(),
         doc="Configuration for requiring confirmation on points reporting",
     )
+    rules = Column(String(255), nullable=True, index=True, doc="Additional rules for this game")
 
     posts = relationship(
         "Post",
@@ -196,15 +226,39 @@ class Game(Base):
             rows = DatabaseSession.query(Play.user_xid).filter(Play.game_id == self.id)
         return [int(row[0]) for row in rows]
 
+    @property
+    def player_pins(self) -> dict[int, str | None]:
+        from spellbot.database import DatabaseSession
+
+        from . import Play
+
+        plays = DatabaseSession.query(Play).filter(Play.game_id == self.id)
+        return {
+            play.user_xid: play.pin if self.guild.enable_mythic_track else None for play in plays
+        }
+
+    @property
+    def player_names(self) -> dict[int, str | None]:
+        from spellbot.database import DatabaseSession
+
+        from . import Play, User
+
+        rows = (
+            DatabaseSession.query(Play.user_xid, User.name)
+            .join(User, User.xid == Play.user_xid)
+            .filter(Play.game_id == self.id)
+        )
+        return {row[0]: row[1] for row in rows}
+
     @property
     def started_at_timestamp(self) -> int:
         assert self.started_at is not None
-        return int(cast(datetime, self.started_at).replace(tzinfo=tz.UTC).timestamp())
+        return int(cast("datetime", self.started_at).replace(tzinfo=tz.UTC).timestamp())
 
     @property
     def updated_at_timestamp(self) -> int:
         assert self.updated_at is not None
-        return int(cast(datetime, self.updated_at).replace(tzinfo=tz.UTC).timestamp())
+        return int(cast("datetime", self.updated_at).replace(tzinfo=tz.UTC).timestamp())
 
     def show_links(self, dm: bool = False) -> bool:
         return True if dm else self.guild.show_links
@@ -213,43 +267,92 @@ class Game(Base):
     def embed_title(self) -> str:
         if self.status == GameStatus.STARTED.value:
             return "**Your game is ready!**"
-        remaining = int(cast(int, self.seats)) - len(self.players)
+        remaining = int(cast("int", self.seats)) - len(self.players)
         plural = "s" if remaining > 1 else ""
         return f"**Waiting for {remaining} more player{plural} to join...**"
 
-    def embed_description(self, dm: bool = False) -> str:  # noqa: C901,PLR0912
+    @tracer.wrap()
+    def embed_description(  # noqa: C901,PLR0912,PLR0915
+        self,
+        *,
+        guild: discord.Guild | None,
+        dm: bool = False,
+        suggested_vc: VoiceChannelSuggestion | None = None,
+        rematch: bool = False,
+    ) -> str:
+        if span := tracer.current_span():  # pragma: no cover
+            span.set_tags(
+                {
+                    "game_id": str(self.id),
+                    "dm": str(dm),
+                    "already_picked": str(suggested_vc.already_picked if suggested_vc else None),
+                    "random_empty": str(suggested_vc.random_empty if suggested_vc else None),
+                }
+            )
+
         description = ""
         if self.guild.notice:
             description += f"{self.guild.notice}\n\n"
+        effective_service = int(self.service)
+        if effective_service == GameService.SPELLTABLE.value and not settings.ENABLE_SPELLTABLE:
+            effective_service = GameService.TABLE_STREAM.value
         if self.status == GameStatus.PENDING.value:
-            if self.service == GameService.SPELLTABLE.value:
+            if effective_service == GameService.SPELLTABLE.value:
                 description += "_A SpellTable link will be created when all players have joined._"
-            elif self.service == GameService.NOT_ANY.value:
+            elif effective_service == GameService.TABLE_STREAM.value:
+                description += "_A Table Stream link will be created when all players have joined._"
+            elif effective_service == GameService.NOT_ANY.value:
                 description += "_Please contact the players in your game to organize this game._"
             else:
-                description += f"_Please use {GameService(self.service)} for this game._"
+                description += f"_Please use {GameService(effective_service)} for this game._"
         else:
             if self.show_links(dm):
-                if self.spelltable_link:
+                if rematch:
                     description += (
-                        f"[Join your SpellTable game now!]({self.spelltable_link})"
-                        f" (or [spectate this game]({self.spectate_link}))"
+                        "This is a rematch of a previous game. "
+                        "Please continue using the same game lobby and voice channel."
                     )
-                elif self.service == GameService.SPELLTABLE.value:
+                elif self.spelltable_link:
+                    description += (
+                        f"# [Join your {GameService(effective_service)} game now!]"
+                        f"({self.spelltable_link})"
+                    )
+                    # Spectate links do not seem to work anymore on SpellTable.
+                    # if effective_service == GameService.SPELLTABLE.value:
+                    #     description += f" (or [spectate this game]({self.spectate_link}))"
+                elif effective_service == GameService.SPELLTABLE.value:
                     description += (
                         "Sorry but SpellBot was unable to create a SpellTable link"
                         " for this game. Please go to [SpellTable]"
                         "(https://spelltable.wizards.com/) to create one."
                     )
-                elif self.service != GameService.NOT_ANY.value:
-                    description += f"Please use {GameService(self.service)} to play this game."
+                elif effective_service == GameService.TABLE_STREAM.value:
+                    description += (
+                        "Sorry but SpellBot was unable to create a Table Stream link"
+                        " for this game. Please go to [Table Stream]"
+                        "(https://table-stream.com/) to create one."
+                    )
+                elif effective_service != GameService.NOT_ANY.value:
+                    description += f"Please use {GameService(effective_service)} to play this game."
                 else:
                     description += "Contact the other players in your game to organize this match."
-
+                if self.password:
+                    description += f"\n\nPassword: `{self.password}`"
                 if self.voice_xid:
-                    description += f"\n\nJoin your voice chat now: <#{self.voice_xid}>"
+                    description += f"\n\n## Join your voice chat now: <#{self.voice_xid}>"
                 if self.voice_invite_link:
                     description += f"\nOr use this voice channel invite: {self.voice_invite_link}"
+                if suggested_vc:
+                    if suggested_vc.already_picked is not None:
+                        description += (
+                            "\n\n## Your pod is already using a voice channel, "
+                            f"join them now: <#{suggested_vc.already_picked}>!\n{HR}"
+                        )
+                    elif suggested_vc.random_empty is not None:
+                        description += (
+                            "\n\n## Please consider using this available voice channel: "
+                            f"<#{suggested_vc.random_empty}>.\n{HR}"
+                        )
             else:
                 description += "Please check your Direct Messages for your game details."
             if dm:
@@ -273,10 +376,11 @@ class Game(Base):
         placeholders = {
             "game_id": str(self.id),
             "game_format": self.format_name,
+            "game_bracket": self.bracket_name,
             "game_start": game_start,
         }
         for i, player in enumerate(self.players):
-            placeholders[f"player_name_{i+1}"] = cast(str, player.name)
+            placeholders[f"player_name_{i + 1}"] = cast("str", player.name)
         return placeholders
 
     def apply_placeholders(self, placeholders: dict[str, str], text: str) -> str:
@@ -290,7 +394,7 @@ class Game(Base):
         for player in self.players:
             points_str = ""
             if self.status == GameStatus.STARTED.value:
-                points = player.points(cast(int, self.id))
+                points = player.points(cast("int", self.id))
                 if points is not None and points[0] is not None:
                     points_value: int = points[0]
                     points_confirmed: bool = points[1]
@@ -338,6 +442,10 @@ class Game(Base):
     def format_name(self) -> str:
         return str(GameFormat(self.format))
 
+    @property
+    def bracket_name(self) -> str:
+        return str(GameBracket(self.bracket))
+
     @property
     def confirmed(self) -> bool:
         from spellbot.database import DatabaseSession
@@ -355,20 +463,34 @@ class Game(Base):
         )
         return player_count == confirmed_count
 
-    def to_embed(self, dm: bool = False) -> discord.Embed:
-        embed = discord.Embed(title=self.embed_title)
-        embed.set_thumbnail(
-            url=settings.QUEER_THUMB_URL if settings.queer(self.guild_xid) else settings.THUMB_URL
+    def to_embed(
+        self,
+        *,
+        guild: discord.Guild | None = None,
+        dm: bool = False,
+        suggested_vc: VoiceChannelSuggestion | None = None,
+        rematch: bool = False,
+    ) -> discord.Embed:
+        embed = discord.Embed(title="**Rematch Game!**" if rematch else self.embed_title)
+        embed.set_thumbnail(url=settings.thumb(self.guild_xid))
+        embed.description = self.embed_description(
+            guild=guild,
+            dm=dm,
+            suggested_vc=suggested_vc,
+            rematch=rematch,
         )
-        embed.description = self.embed_description(dm)
+        if self.rules:
+            embed.add_field(name="⚠️ Additional Rules:", value=self.rules, inline=False)
         if self.players:
             embed.add_field(name="Players", value=self.embed_players, inline=False)
         embed.add_field(name="Format", value=self.format_name)
+        if self.bracket != GameBracket.NONE.value:
+            embed.add_field(name="Bracket", value=self.bracket_name)
         if self.started_at:
             embed.add_field(name="Started at", value=f"<t:{self.started_at_timestamp}>")
         else:
             embed.add_field(name="Updated at", value=f"<t:{self.updated_at_timestamp}>")
-        if self.service != GameService.SPELLTABLE.value:
+        if self.service != GameService.SPELLTABLE.value and not rematch:
             embed.add_field(
                 name="Service",
                 value=str(GameService(self.service)),
@@ -382,6 +504,12 @@ class Game(Base):
             )
         else:
             embed.color = discord.Color(settings.EMPTY_EMBED_COLOR)
+        if suggested_vc and (suggested_vc_xid := suggested_vc.get()):
+            embed.add_field(
+                name="🔊 Suggested Voice Channel",
+                value=f"<#{suggested_vc_xid}>",
+                inline=False,
+            )
         embed.add_field(
             name="Support SpellBot",
             value=(
@@ -408,10 +536,16 @@ class Game(Base):
             "seats": self.seats,
             "status": self.status,
             "format": self.format,
+            "bracket": self.bracket,
             "service": self.service,
             "spelltable_link": self.spelltable_link,
             "spectate_link": self.spectate_link,
             "jump_links": self.jump_links,
             "confirmed": self.confirmed,
             "requires_confirmation": self.channel.require_confirmation,
+            "password": self.password,
+            "rules": self.rules,
         }
+
+
+MAX_RULES_LENGTH: int = Game.rules.property.columns[0].type.length  # type: ignore
