@@ -1,10 +1,10 @@
 from __future__ import annotations
 
+from .apps import AppsService
 from .awards import AwardsService, NewAward
 from .channels import ChannelsService
 from .games import GamesService
 from .guilds import GuildsService
-from .mirrors import MirrorsService
 from .plays import PlaysService
 from .users import UsersService
 from .verifies import VerifiesService
@@ -17,19 +17,19 @@ class ServicesRegistry:
         self.channels = ChannelsService()
         self.games = GamesService()
         self.guilds = GuildsService()
-        self.mirrors = MirrorsService()
         self.plays = PlaysService()
         self.users = UsersService()
         self.verifies = VerifiesService()
         self.watches = WatchesService()
+        self.apps = AppsService()
 
 
 __all__ = [
+    "AppsService",
     "AwardsService",
     "ChannelsService",
     "GamesService",
     "GuildsService",
-    "MirrorsService",
     "NewAward",
     "PlaysService",
     "ServicesRegistry",
