@@ -1,9 +1,8 @@
 from __future__ import annotations
 
-from datetime import datetime
+from datetime import UTC, datetime
 from typing import TYPE_CHECKING
 
-import pytz
 from asgiref.sync import sync_to_async
 from sqlalchemy.dialects.postgresql import insert
 from sqlalchemy.sql.expression import update
@@ -26,7 +25,7 @@ class ChannelsService:
             "xid": channel.id,
             "guild_xid": channel.guild.id,
             "name": name,
-            "updated_at": datetime.now(tz=pytz.utc),
+            "updated_at": datetime.now(tz=UTC),
         }
         upsert = insert(Channel).values(**values)
         upsert = upsert.on_conflict_do_update(
@@ -55,7 +54,7 @@ class ChannelsService:
     @sync_to_async()
     def set_default_seats(self, xid: int, seats: int) -> None:
         query = (
-            update(Channel)
+            update(Channel)  # type: ignore
             .where(Channel.xid == xid)
             .values(default_seats=seats)
             .execution_options(synchronize_session=False)
@@ -66,7 +65,7 @@ class ChannelsService:
     @sync_to_async()
     def set_default_format(self, xid: int, format: int) -> None:
         query = (
-            update(Channel)
+            update(Channel)  # type: ignore
             .where(Channel.xid == xid)
             .values(default_format=format)
             .execution_options(synchronize_session=False)
@@ -74,10 +73,21 @@ class ChannelsService:
         DatabaseSession.execute(query)
         DatabaseSession.commit()
 
+    @sync_to_async()
+    def set_default_bracket(self, xid: int, bracket: int) -> None:
+        query = (
+            update(Channel)  # type: ignore
+            .where(Channel.xid == xid)
+            .values(default_bracket=bracket)
+            .execution_options(synchronize_session=False)
+        )
+        DatabaseSession.execute(query)
+        DatabaseSession.commit()
+
     @sync_to_async()
     def set_default_service(self, xid: int, service: int) -> None:
         query = (
-            update(Channel)
+            update(Channel)  # type: ignore
             .where(Channel.xid == xid)
             .values(default_service=service)
             .execution_options(synchronize_session=False)
@@ -88,7 +98,7 @@ class ChannelsService:
     @sync_to_async()
     def set_auto_verify(self, xid: int, setting: bool) -> None:
         query = (
-            update(Channel)
+            update(Channel)  # type: ignore
             .where(Channel.xid == xid)
             .values(auto_verify=setting)
             .execution_options(synchronize_session=False)
@@ -99,7 +109,7 @@ class ChannelsService:
     @sync_to_async()
     def set_verified_only(self, xid: int, setting: bool) -> None:
         query = (
-            update(Channel)
+            update(Channel)  # type: ignore
             .where(Channel.xid == xid)
             .values(verified_only=setting)
             .execution_options(synchronize_session=False)
@@ -110,7 +120,7 @@ class ChannelsService:
     @sync_to_async()
     def set_unverified_only(self, xid: int, setting: bool) -> None:
         query = (
-            update(Channel)
+            update(Channel)  # type: ignore
             .where(Channel.xid == xid)
             .values(unverified_only=setting)
             .execution_options(synchronize_session=False)
@@ -126,7 +136,7 @@ class ChannelsService:
         else:
             motd = ""
         query = (
-            update(Channel)
+            update(Channel)  # type: ignore
             .where(Channel.xid == xid)
             .values(motd=motd)
             .execution_options(synchronize_session=False)
@@ -143,7 +153,7 @@ class ChannelsService:
         else:
             extra = ""
         query = (
-            update(Channel)
+            update(Channel)  # type: ignore
             .where(Channel.xid == xid)
             .values(extra=extra)
             .execution_options(synchronize_session=False)
@@ -157,7 +167,7 @@ class ChannelsService:
         max_len = Channel.voice_category.property.columns[0].type.length  # type: ignore
         name = value[:max_len]
         query = (
-            update(Channel)
+            update(Channel)  # type: ignore
             .where(Channel.xid == xid)
             .values(voice_category=name)
             .execution_options(synchronize_session=False)
@@ -169,7 +179,7 @@ class ChannelsService:
     @sync_to_async()
     def set_delete_expired(self, xid: int, value: bool) -> bool:
         query = (
-            update(Channel)
+            update(Channel)  # type: ignore
             .where(Channel.xid == xid)
             .values(delete_expired=value)
             .execution_options(synchronize_session=False)
@@ -181,7 +191,7 @@ class ChannelsService:
     @sync_to_async()
     def set_show_points(self, xid: int, value: bool) -> bool:
         query = (
-            update(Channel)
+            update(Channel)  # type: ignore
             .where(Channel.xid == xid)
             .values(show_points=value)
             .execution_options(synchronize_session=False)
@@ -190,22 +200,23 @@ class ChannelsService:
         DatabaseSession.commit()
         return value
 
-    @sync_to_async()
-    def set_require_confirmation(self, xid: int, value: bool) -> bool:
-        query = (
-            update(Channel)
-            .where(Channel.xid == xid)
-            .values(require_confirmation=value)
-            .execution_options(synchronize_session=False)
-        )
-        DatabaseSession.execute(query)
-        DatabaseSession.commit()
-        return value
+    # TODO: Refactor how confirmation/points/ELO works.
+    # @sync_to_async()
+    # def set_require_confirmation(self, xid: int, value: bool) -> bool:
+    #     query = (
+    #         update(Channel)
+    #         .where(Channel.xid == xid)
+    #         .values(require_confirmation=value)
+    #         .execution_options(synchronize_session=False)
+    #     )
+    #     DatabaseSession.execute(query)
+    #     DatabaseSession.commit()
+    #     return value
 
     @sync_to_async()
     def set_voice_invite(self, xid: int, value: bool) -> bool:
         query = (
-            update(Channel)
+            update(Channel)  # type: ignore
             .where(Channel.xid == xid)
             .values(voice_invite=value)
             .execution_options(synchronize_session=False)
