@@ -1,14 +1,11 @@
 from __future__ import annotations
 
 import logging
-from collections import defaultdict
-from datetime import datetime, timedelta
-from itertools import permutations
+from datetime import UTC, datetime, timedelta
 from typing import TYPE_CHECKING, cast
 
-import pytz
 from asgiref.sync import sync_to_async
-from ddtrace import tracer
+from ddtrace.trace import tracer
 from sqlalchemy import func, select, update
 from sqlalchemy.dialects.postgresql import insert
 from sqlalchemy.orm import aliased
@@ -22,14 +19,10 @@ from spellbot.models import (
     Game,
     GameDict,
     GameStatus,
-    MirrorDict,
     Play,
-    PlayDict,
     Post,
     Queue,
     QueueDict,
-    Record,
-    RecordDict,
     UserAward,
     Watch,
 )
@@ -38,9 +31,12 @@ from spellbot.settings import settings
 if TYPE_CHECKING:
     import discord
 
+    from spellbot.operations import VoiceChannelSuggestion
+
+
 logger = logging.getLogger(__name__)
 
-MAX_SPELLTABLE_LINK_LEN = Game.spelltable_link.property.columns[0].type.length  # type: ignore
+MAX_GAME_LINK_LEN = Game.spelltable_link.property.columns[0].type.length  # type: ignore
 
 
 class GamesService:
@@ -49,7 +45,7 @@ class GamesService:
     @sync_to_async()
     @tracer.wrap()
     def select(self, game_id: int) -> GameDict | None:
-        self.game = DatabaseSession.query(Game).get(game_id)
+        self.game = DatabaseSession.get(Game, game_id)
         return self.game.to_dict() if self.game else None
 
     @sync_to_async()
@@ -88,36 +84,6 @@ class GamesService:
         )
         return self.game.to_dict() if self.game else None
 
-    @sync_to_async()
-    @tracer.wrap()
-    def get_plays(self) -> dict[int, PlayDict]:
-        assert self.game
-        plays = DatabaseSession.query(Play).filter(Play.game_id == self.game.id).all()
-        return {play.user_xid: play.to_dict() for play in plays}
-
-    @sync_to_async()
-    @tracer.wrap()
-    def get_record(self, guild_xid: int, channel_xid: int, user_xid: int) -> RecordDict:
-        record = (
-            DatabaseSession.query(Record)
-            .filter(
-                Record.guild_xid == guild_xid,
-                Record.channel_xid == channel_xid,
-                Record.user_xid == user_xid,
-            )
-            .one_or_none()
-        )
-        if not record:
-            record = Record(
-                guild_xid=guild_xid,
-                channel_xid=channel_xid,
-                user_xid=user_xid,
-                elo=1500,
-            )
-            DatabaseSession.add(record)
-            DatabaseSession.commit()
-        return record.to_dict()
-
     @sync_to_async()
     @tracer.wrap()
     def add_player(self, player_xid: int) -> None:
@@ -144,9 +110,9 @@ class GamesService:
 
         # This operation should "dirty" the Game, so we need to update its updated_at.
         query = (
-            update(Game)
+            update(Game)  # type: ignore
             .where(Game.id == self.game.id)
-            .values(updated_at=datetime.now(tz=pytz.utc))
+            .values(updated_at=datetime.now(tz=UTC))
             .execution_options(synchronize_session=False)
         )
         DatabaseSession.execute(query)
@@ -162,12 +128,12 @@ class GamesService:
         author_xid: int,
         friends: list[int],
         seats: int,
+        rules: str | None,
         format: int,
+        bracket: int,
         service: int,
         create_new: bool = False,
-        mirrors: list[MirrorDict] | None = None,
     ) -> bool:
-        mirrors = mirrors or []
         existing: Game | None = None
         if not create_new:
             existing = self._find_existing(
@@ -176,9 +142,10 @@ class GamesService:
                 author_xid=author_xid,
                 friends=friends,
                 seats=seats,
+                rules=rules,
                 format=format,
+                bracket=bracket,
                 service=service,
-                mirrors=mirrors,
             )
 
         new: bool
@@ -187,12 +154,14 @@ class GamesService:
             game = existing
             new = False
         else:
-            channel = DatabaseSession.query(Channel).get(channel_xid)
+            channel = DatabaseSession.get(Channel, channel_xid)
             game = Game(
                 guild_xid=guild_xid,
                 channel_xid=channel_xid,
                 seats=seats,
+                rules=rules,
                 format=format,
+                bracket=bracket,
                 service=service,
                 requires_confirmation=channel.require_confirmation,
             )
@@ -230,39 +199,29 @@ class GamesService:
         author_xid: int,
         friends: list[int],
         seats: int,
+        rules: str | None,
         format: int,
+        bracket: int,
         service: int,
-        mirrors: list[MirrorDict] | None = None,
     ) -> Game | None:
-        mirrors = mirrors or []
         required_seats = 1 + len(friends)
 
-        guild_channel_filter = or_(
-            *[
-                and_(
-                    Game.guild_xid == gid,
-                    Game.channel_xid == cid,
-                )
-                for gid, cid in [
-                    (guild_xid, channel_xid),
-                    *[(m["to_guild_xid"], m["to_channel_xid"]) for m in mirrors],
-                ]
-            ]
-        )
-
         player_count = count(Queue.user_xid).over(partition_by=Game.id)
         inner = (
             select(
-                Game,
+                Game,  # type: ignore
                 Queue.user_xid,
                 player_count.label("player_count"),  # type: ignore
             )
-            .join(Queue, isouter=True)
+            .join(Queue, isouter=True)  # type: ignore
             .filter(  # type: ignore
                 and_(
-                    guild_channel_filter,
+                    Game.guild_xid == guild_xid,
+                    Game.channel_xid == channel_xid,
                     Game.seats == seats,
+                    Game.rules == rules,
                     Game.format == format,
+                    Game.bracket == bracket,
                     Game.service == service,
                     Game.status == GameStatus.PENDING.value,
                     Game.deleted_at.is_(None),
@@ -310,9 +269,16 @@ class GamesService:
 
     @sync_to_async()
     @tracer.wrap()
-    def to_embed(self, dm: bool = False) -> discord.Embed:
+    def to_embed(
+        self,
+        *,
+        guild: discord.Guild | None,
+        dm: bool = False,
+        suggested_vc: VoiceChannelSuggestion | None = None,
+        rematch: bool = False,
+    ) -> discord.Embed:
         assert self.game
-        return self.game.to_embed(dm)
+        return self.game.to_embed(guild=guild, dm=dm, suggested_vc=suggested_vc, rematch=rematch)
 
     @sync_to_async()
     @tracer.wrap()
@@ -355,22 +321,23 @@ class GamesService:
 
     @sync_to_async()
     @tracer.wrap()
-    def make_ready(self, spelltable_link: str | None) -> int:
+    def make_ready(self, game_link: str | None, password: str | None) -> int:
         assert self.game
-        assert len(spelltable_link or "") <= MAX_SPELLTABLE_LINK_LEN
+        assert len(game_link or "") <= MAX_GAME_LINK_LEN
         queues: list[QueueDict] = [
             queue.to_dict()
             for queue in DatabaseSession.query(Queue).filter(Queue.game_id == self.game.id).all()
         ]
 
         # update game's state
-        self.game.spelltable_link = spelltable_link
+        self.game.spelltable_link = game_link  # column is "spelltable_link" for legacy reasons
+        self.game.password = password
         self.game.status = GameStatus.STARTED.value
-        self.game.started_at = datetime.now(tz=pytz.utc)
+        self.game.started_at = datetime.now(tz=UTC)
 
         if not queues:  # Not sure this is possible, but just in case.
             DatabaseSession.commit()
-            return cast(int, self.game.id)
+            return cast("int", self.game.id)
 
         # upsert into plays
         DatabaseSession.execute(
@@ -410,7 +377,7 @@ class GamesService:
         )
 
         DatabaseSession.commit()
-        return cast(int, self.game.id)
+        return cast("int", self.game.id)
 
     @sync_to_async()
     @tracer.wrap()
@@ -418,6 +385,18 @@ class GamesService:
         assert self.game
         return self.game.player_xids
 
+    @sync_to_async()
+    @tracer.wrap()
+    def player_pins(self) -> dict[int, str | None]:
+        assert self.game
+        return self.game.player_pins
+
+    @sync_to_async()
+    @tracer.wrap()
+    def player_names(self) -> dict[int, str | None]:
+        assert self.game
+        return self.game.player_names
+
     @sync_to_async()
     @tracer.wrap()
     def watch_notes(self, player_xids: list[int]) -> dict[int, str | None]:
@@ -432,7 +411,7 @@ class GamesService:
             )
             .all()
         )
-        return {cast(int, watch.user_xid): cast(str | None, watch.note) for watch in watched}
+        return {cast("int", watch.user_xid): cast("str | None", watch.note) for watch in watched}
 
     @sync_to_async()
     @tracer.wrap()
@@ -447,12 +426,12 @@ class GamesService:
     def filter_blocked_list(self, author_xid: int, other_xids: list[int]) -> list[int]:
         """Given an author, filters out any blocked players from a list of others."""
         users_author_has_blocked = [
-            cast(int, row.blocked_user_xid)
+            cast("int", row.blocked_user_xid)
             for row in DatabaseSession.query(Block).filter(Block.user_xid == author_xid)
             if row
         ]
         users_who_blocked_author_or_other = [
-            cast(int, row.user_xid)
+            cast("int", row.user_xid)
             for row in DatabaseSession.query(Block).filter(
                 Block.blocked_user_xid.in_([author_xid, *other_xids]),
             )
@@ -547,9 +526,9 @@ class GamesService:
     @tracer.wrap()
     def confirm_points(self, player_xid: int) -> datetime:
         assert self.game
-        confirmed_at = datetime.now(tz=pytz.utc)
+        confirmed_at = datetime.now(tz=UTC)
         query = (
-            update(Play)
+            update(Play)  # type: ignore
             .where(
                 and_(
                     Play.game_id == self.game.id,
@@ -562,76 +541,6 @@ class GamesService:
         DatabaseSession.commit()
         return confirmed_at
 
-    @sync_to_async()
-    @tracer.wrap()
-    def update_records(self, plays: dict[int, PlayDict]) -> None:  # noqa: C901
-        assert self.game
-        guild_xid = self.game.guild_xid
-        channel_xid = self.game.channel_xid
-        records: dict[int, Record] = {
-            r.user_xid: r
-            for r in DatabaseSession.query(Record).filter(
-                Record.guild_xid == guild_xid,
-                Record.channel_xid == channel_xid,
-            )
-        }
-        if len(records) != len(plays):
-            # some players may not have Record objects for this guild/channel yet, so
-            # we must create them now (giving each player their default ELO scores).
-            for user_xid in plays:
-                if user_xid not in records:
-                    record = Record(
-                        guild_xid=guild_xid,
-                        channel_xid=channel_xid,
-                        user_xid=user_xid,
-                        elo=1500,
-                    )
-                    DatabaseSession.add(record)
-                    records[user_xid] = record
-
-        assert len(records) == len(plays)
-
-        def place(play: PlayDict) -> int:
-            points = play["points"] or 0
-            if points >= 3:  # first place
-                return 1
-            if points > 0:  # second place
-                return 2
-            return 3  # last place
-
-        def calc_s(play1: PlayDict, play2: PlayDict):
-            if place(play1) < place(play2):
-                return 1.0
-            if place(play1) == place(play2):
-                return 0.5
-            return 0.0
-
-        def calc_ea(rec1: Record, rec2: Record):
-            return 1 / (1.0 + pow(10.0, (rec2.elo - rec1.elo) / 400.0))
-
-        def calc_elo_change(play1: PlayDict, rec1: Record, play2: PlayDict, rec2: Record, k: int):
-            s = calc_s(play1, play2)
-            ea = calc_ea(rec1, rec2)
-            return round(k * (s - ea))
-
-        n = len(records)
-        k = 32  # growth rate
-        player_to_player = list(permutations(range(n), 2))
-
-        elo_changes = defaultdict(int)
-        plays_list = sorted(plays.values(), key=lambda p: p["user_xid"])
-        records_list = sorted(records.values(), key=lambda r: r.user_xid)
-        for i, j in player_to_player:
-            elo_changes[i] += calc_elo_change(
-                plays_list[i], records_list[i], plays_list[j], records_list[j], k
-            )
-        for i in range(n):
-            before = records_list[i].elo
-            after = before + elo_changes[i]
-            print(f"ELO {records_list[i].user_xid}, {before} -> {after} ...")  # noqa: T201
-            records_list[i].elo = after  # type: ignore
-        DatabaseSession.commit()
-
     @sync_to_async()
     @tracer.wrap()
     def to_dict(self) -> GameDict:
@@ -641,7 +550,7 @@ class GamesService:
     @sync_to_async()
     @tracer.wrap()
     def inactive_games(self) -> list[GameDict]:
-        limit = datetime.now(tz=pytz.utc) - timedelta(minutes=settings.EXPIRE_TIME_M)
+        limit = datetime.now(tz=UTC) - timedelta(minutes=settings.EXPIRE_TIME_M)
         records = (
             DatabaseSession.query(Game)
             .join(Queue, isouter=True)
@@ -663,7 +572,9 @@ class GamesService:
     @tracer.wrap()
     def delete_games(self, game_ids: list[int]) -> int:
         query = (
-            update(Game).where(Game.id.in_(game_ids)).values(deleted_at=datetime.now(tz=pytz.utc))
+            update(Game)  # type: ignore
+            .where(Game.id.in_(game_ids))
+            .values(deleted_at=datetime.now(tz=UTC))
         )
         DatabaseSession.execute(query)
         dequeued = (
@@ -688,8 +599,25 @@ class GamesService:
     def dequeue_players(self, player_xids: list[int]) -> list[int]:
         """Remove the given players from any queues that they're in; returns changed game ids."""
         queues = DatabaseSession.query(Queue).filter(Queue.user_xid.in_(player_xids)).all()
-        game_ids = {cast(int, queue.game_id) for queue in queues}
+        game_ids = {cast("int", queue.game_id) for queue in queues}
         for queue in queues:
             DatabaseSession.delete(queue)
         DatabaseSession.commit()
         return list(game_ids)
+
+    @sync_to_async()
+    @tracer.wrap()
+    def select_last_game(self, user_xid: int, guild_xid: int) -> GameDict | None:
+        self.game = (
+            DatabaseSession.query(Game)
+            .filter(
+                Game.guild_xid == guild_xid,
+                Game.status == GameStatus.STARTED.value,
+                Game.deleted_at.is_(None),
+                Play.user_xid == user_xid,
+            )
+            .join(Play)
+            .order_by(Game.created_at.desc())
+            .first()
+        )
+        return self.game.to_dict() if self.game else None
