@@ -1,10 +1,9 @@
 from __future__ import annotations
 
-from datetime import datetime
+from datetime import UTC, datetime
 from os import getenv
 from typing import TYPE_CHECKING
 
-import pytz
 from discord import Object
 
 from .environment import running_in_pytest
@@ -15,38 +14,45 @@ if TYPE_CHECKING:
 
 class Settings:
     __slots__ = (
-        "guild_xid",
-        "CONTENT_ROOT",
-        "BOT_TOKEN",
-        "BOT_APPLICATION_ID",
-        "PORT",
-        "HOST",
-        "DEBUG_GUILD",
+        "ADMIN_ROLE",
         "API_BASE_URL",
+        "BOT_APPLICATION_ID",
+        "BOT_INVITE_LINK",
+        "BOT_TOKEN",
+        "CONTENT_ROOT",
+        "DATABASE_ECHO",
+        "DATABASE_URL",
         "DD_API_KEY",
         "DD_APP_KEY",
         "DD_TRACE_ENABLED",
-        "DATABASE_URL",
-        "SPELLTABLE_ROOT",
-        "SPELLTABLE_CREATE",
-        "SPELLTABLE_AUTH_KEY",
-        "BOT_INVITE_LINK",
-        "INFO_EMBED_COLOR",
-        "STARTED_EMBED_COLOR",
-        "PENDING_EMBED_COLOR",
+        "DEBUG_GUILD",
+        "DONATE_LINK",
         "EMPTY_EMBED_COLOR",
-        "DATABASE_ECHO",
-        "ADMIN_ROLE",
-        "MOD_PREFIX",
-        "MAX_PENDING_GAMES",
-        "VOICE_GRACE_PERIOD_M",
-        "VOICE_AGE_LIMIT_H",
-        "VOICE_CLEANUP_LOOP_M",
-        "VOICE_CLEANUP_BATCH",
+        "ENABLE_SPELLTABLE",
         "EXPIRE_GAMES_LOOP_M",
         "EXPIRE_TIME_M",
+        "HOST",
+        "INFO_EMBED_COLOR",
+        "MAX_PENDING_GAMES",
+        "MOD_PREFIX",
+        "PENDING_EMBED_COLOR",
+        "PORT",
+        "REDISCLOUD_URL",
+        "SPELLTABLE_AUTH_KEY",
+        "SPELLTABLE_CREATE",
+        "SPELLTABLE_PASSES",
+        "SPELLTABLE_ROOT",
+        "SPELLTABLE_USERS",
+        "STARTED_EMBED_COLOR",
         "SUBSCRIBE_LINK",
-        "DONATE_LINK",
+        "TABLESTREAM_AUTH_KEY",
+        "TABLESTREAM_CREATE",
+        "TABLESTREAM_ROOT",
+        "VOICE_AGE_LIMIT_H",
+        "VOICE_CLEANUP_BATCH",
+        "VOICE_CLEANUP_LOOP_M",
+        "VOICE_GRACE_PERIOD_M",
+        "guild_xid",
     )
 
     def __init__(self, guild_xid: int | None = None) -> None:
@@ -78,13 +84,29 @@ class Settings:
         if database_url.startswith("postgres://"):  # pragma: no cover
             # SQLAlchemy 1.4.x removed support for the postgres:// URI scheme
             database_url = database_url.replace("postgres://", "postgresql://", 1)
+        if database_url.startswith("postgresql://"):  # pragma: no cover
+            # Ensure that we're asking for the psycopg3+ driver (and not psycopg2)
+            database_url = database_url.replace("postgresql://", "postgresql+psycopg://", 1)
         self.DATABASE_URL = database_url
 
+        # cache
+        self.REDISCLOUD_URL = getenv("REDISCLOUD_URL")
+
         # spelltable
+        self.ENABLE_SPELLTABLE = getenv("ENABLE_SPELLTABLE", "true").lower() == "true"
         self.SPELLTABLE_ROOT = "https://us-central1-magic-night-30324.cloudfunctions.net"
         self.SPELLTABLE_CREATE = f"{self.SPELLTABLE_ROOT}/createGame"
         self.SPELLTABLE_AUTH_KEY = getenv("SPELLTABLE_AUTH_KEY")
 
+        # spelltable headless
+        self.SPELLTABLE_USERS = getenv("SPELLTABLE_USERS")
+        self.SPELLTABLE_PASSES = getenv("SPELLTABLE_PASSES")
+
+        # tablestream
+        self.TABLESTREAM_ROOT = "https://api.table-stream.com"
+        self.TABLESTREAM_CREATE = f"{self.TABLESTREAM_ROOT}/create-room"
+        self.TABLESTREAM_AUTH_KEY = getenv("TABLESTREAM_AUTH_KEY")
+
         # configuration
         self.BOT_INVITE_LINK = (
             r"https://discordapp.com/api/oauth2/authorize"
@@ -110,16 +132,12 @@ class Settings:
         self.EXPIRE_TIME_M = 45  # 45 minutes
 
     def workaround_over_eager_caching(self, url: str) -> str:
-        return f"{url}?{datetime.now(tz=pytz.utc).date().strftime('%Y-%m-%d')}"
+        return f"{url}?{datetime.now(tz=UTC).date().strftime('%Y-%m-%d')}"
 
-    def queer(self, guild_xid: int | None) -> bool:
-        return (
-            guild_xid
-            in [
-                757455940009328670,  # Oath of the Gaywatch
-                699775410082414733,  # Development
-            ]
-            or datetime.now(tz=pytz.utc).month == 6
+    @property
+    def ICO_URL(self) -> str:
+        return self.workaround_over_eager_caching(
+            f"{self.CONTENT_ROOT}/spellbot/main/spellbot-sm.png",
         )
 
     @property
@@ -128,6 +146,16 @@ class Settings:
             f"{self.CONTENT_ROOT}/spellbot/main/spellbot.png",
         )
 
+    def queer(self, guild_xid: int | None) -> bool:  # pragma: no cover
+        return (
+            guild_xid
+            in [
+                757455940009328670,  # Oath of the Gaywatch
+                # 699775410082414733,  # Development
+            ]
+            or datetime.now(tz=UTC).month == 6
+        )
+
     @property
     def QUEER_THUMB_URL(self) -> str:  # pragma: no cover
         return self.workaround_over_eager_caching(
@@ -135,19 +163,57 @@ class Settings:
             "149257079-e3efe74f-482b-4410-a0ea-dd988a4d3c63.png",
         )
 
+    def black(self, guild_xid: int | None) -> bool:  # pragma: no cover
+        return datetime.now(tz=UTC).month == 2
+
     @property
-    def ICO_URL(self) -> str:
+    def BLACK_THUMB_URL(self) -> str:  # pragma: no cover
         return self.workaround_over_eager_caching(
-            f"{self.CONTENT_ROOT}/spellbot/main/spellbot-sm.png",
+            "https://github.com/user-attachments/assets/146f40fd-1d8a-4027-937a-b00799ce89bc",
         )
 
     @property
-    def QUEER_ICO_URL(self) -> str:  # pragma: no cover
+    def TRANS_THUMB_URL(self) -> str:  # pragma: no cover
         return self.workaround_over_eager_caching(
-            "https://user-images.githubusercontent.com/1903876/"
-            "149257564-86595c81-82a5-4558-ae40-c03d29a95d1f.png",
+            "https://github.com/user-attachments/assets/cb469ae7-5a31-4855-9510-03762e204b52",
         )
 
+    def trans(self, guild_xid: int | None) -> bool:  # pragma: no cover
+        now = datetime.now(tz=UTC)
+        return now.month == 11 or (now.month == 3 and now.day == 31)
+
+    @property
+    def WOMEN_THUMB_URL(self) -> str:  # pragma: no cover
+        return self.workaround_over_eager_caching(
+            "https://github.com/user-attachments/assets/b66f40cc-d2c7-49eb-880a-c8965d94d412",
+        )
+
+    def women(self, guild_xid: int | None) -> bool:  # pragma: no cover
+        return datetime.now(tz=UTC).month == 3
+
+    @property
+    def AUTISTIC_THUMB_URL(self) -> str:  # pragma: no cover
+        return self.workaround_over_eager_caching(
+            "https://github.com/user-attachments/assets/e2799f26-6873-452a-97a4-e25a64d6d991",
+        )
+
+    def autistic(self, guild_xid: int | None) -> bool:  # pragma: no cover
+        now = datetime.now(tz=UTC)
+        return now.month == 4 and now.day == 2
+
+    def thumb(self, guild_xid: int | None) -> str:  # pragma: no cover
+        if self.autistic(guild_xid):
+            return settings.AUTISTIC_THUMB_URL
+        if settings.trans(guild_xid):
+            return settings.TRANS_THUMB_URL
+        if settings.women(guild_xid):
+            return settings.WOMEN_THUMB_URL
+        if settings.queer(guild_xid):
+            return settings.QUEER_THUMB_URL
+        if settings.black(guild_xid):
+            return settings.BLACK_THUMB_URL
+        return settings.THUMB_URL
+
     @property
     def GUILD_OBJECT(self) -> Snowflake | None:
         return Object(id=self.DEBUG_GUILD) if self.DEBUG_GUILD else None
